/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <sched.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <jutils.h>
#include <gui.h>

#include <config.h>

#define MAX_DEBUG 2

#define FUNC 2 /* se il debug level e' questo
		  ci sono le funzioni chiamate */
#define WARN 1 /* ... blkbblbl */

static int verbosity;
static GUI *gui = NULL;
static FILE *logfd = NULL;

void set_debug(int lev) {
  lev = lev<0 ? 0 : lev;
  lev = lev>MAX_DEBUG ? MAX_DEBUG : lev;
  verbosity = lev;
}

int get_debug() {
  return(verbosity);
}

void set_guimsg(GUI *g) {
  gui = g;
}

void set_logfile(char *file) {
  logfd = fopen(file,"w");
  if(!logfd) {
    error("can't open logfile %s",file);
    error("%s",strerror(errno));
  }
}

void close_logfile() {
  if(logfd) fclose(logfd);
}

void notice(const char *format, ...) {
  char msg[255];
  va_list arg;
  va_start(arg, format);

  vsnprintf(msg, 254, format, arg);

  if(logfd) { fputs(msg,logfd); fputs("\n",logfd); fflush(logfd); }
  else cerr << "[*] " << msg << endl;
  if(gui) gui->set_status(msg);

  va_end(arg);
}

void func(const char *format, ...) {
  if(verbosity>=FUNC) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsnprintf(msg, 254, format, arg);
    if(logfd) { fputs(msg,logfd); fputs("\n",logfd); fflush(logfd); }
    else cerr << "[F] " << msg << endl;
    
    va_end(arg);
  }
}

void error(const char *format, ...) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsnprintf(msg, 254, format, arg);
    if(logfd) { fputs(msg,logfd); fputs("\n",logfd); fflush(logfd); }
    else cerr << "[!] " << msg << endl;
    //    perror(msg);    

    if(gui) gui->set_status(msg);
    va_end(arg);
}

void act(const char *format, ...) {
  char msg[255];
  va_list arg;
  va_start(arg, format);
  
  vsprintf(msg, format, arg);

  if(logfd) { fputs(msg,logfd); fputs("\n",logfd); fflush(logfd); }
  else cerr << " .  " << msg << endl;
  
  va_end(arg);
}

void warning(const char *format, ...) {
  if(verbosity>=WARN) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsprintf(msg, format, arg);

    if(logfd) { fputs(msg,logfd); fputs("\n",logfd); fflush(logfd); }
    else cerr << "[W] " << msg << endl;
  
    va_end(arg);
  }
}

double dtime() {
  struct timeval mytv;
  gettimeofday(&mytv,NULL);
  return((double)mytv.tv_sec+1.0e-6*(double)mytv.tv_usec);
}

void jsleep(int sec, long nsec) {
  timespec timelap;
  timelap.tv_sec = sec;
  timelap.tv_nsec = nsec;
  nanosleep(&timelap,NULL);
}

/* sets the process to "policy" policy,  if max=1 then set at max priority,
   else use min priority */

bool set_rtpriority(bool max) {
  struct sched_param schp;
  /*
   * set the process to realtime privs
   */
  memset(&schp, 0, sizeof(schp));
  
  if(max) 
    schp.sched_priority = sched_get_priority_max(SCHED_RR);
  else
    schp.sched_priority = sched_get_priority_min(SCHED_RR);
  
  if (sched_setscheduler(0, SCHED_RR, &schp) != 0)
    return false;
  else
    return true;
}

void chomp(char *str) {
  size_t len; //, ilen;
  char tmp[MAX_PATH_SIZE], *p = str;
  
  memset(tmp,'\0',MAX_PATH_SIZE);
  
  /* eliminate space and tabs at the beginning */
  while (*p == ' ' || *p == '\t') p++;
  strncpy(tmp, p, MAX_PATH_SIZE);
  
  /* point *p at the end of string */
  len = strlen(tmp); 
  p = &tmp[len-1];
  
  while ((*p == ' ' || *p == '\t' || *p == '\n') && len) {
    *p = '\0'; p--; len--;
  }

  strncpy(str, tmp, MAX_PATH_SIZE);
}

bool resolve(char *host, char *ip) {
  func("resolving %s ...",host);
  struct sockaddr_in server_addr;
  struct hostent *h;
  
  memset(&server_addr, 0, sizeof(struct sockaddr_in));
  server_addr.sin_family = AF_INET;
  
#ifdef HAVE_INET_ATON
  inet_aton(host,&server_addr.sin_addr);
#else
  server_addr.sin_addr.s_addr=inet_addr(host);
#endif
  
  if(!(h = gethostbyname(host))) {
    error("cannot resolve host: %s",host);
    return(false);
  }

  server_addr.sin_family = AF_INET;
  memcpy(&server_addr.sin_addr, h->h_addr, h->h_length);
  sprintf(ip,"%s",
	  inet_ntoa(*(struct in_addr *) &server_addr.sin_addr.s_addr));
  func("%s resolved to %s",host, ip);
  return(true);
}
