/* A NCURSES/CDK TUI (Text User Interface) for MuSE
 * Copyright (C) 2002 Luca 'rubik' Profico <rubik@olografix.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: connectbox.cpp,v 1.14 2003/02/22 21:45:03 rubik Exp $
 *
 */

#include "connectbox.h"
#include "muse_console.h"

ConnectBox::ConnectBox (Stream_mixer *m)
{

	mixer = m ;

	func ("ConnectBox::ConnectBox");

	ybase = (LINES-CONNBOX_HEIGHT)/2;
	xbase = (COLS-CONNBOX_WIDTH)/2;
	subwin = newwin (CONNBOX_HEIGHT, CONNBOX_WIDTH, ybase, xbase);

	warning ("ConnBox pars: %d, %d, %d, %d",xbase, ybase,CONNBOX_WIDTH, CONNBOX_HEIGHT);

	cdkscreen = initCDKScreen (subwin);
	box (subwin, ACS_VLINE, ACS_HLINE);
	wrefresh (subwin);


	/* labels */
	bottom.setscreen (cdkscreen);

	encsetbox.setscreen (cdkscreen);
	//streamsetbox.setscreen (cdkscreen);



	/* load default profiles */

	/* nuova api
	mixer->lame.load_profile("last_active");
	mixer->ice.load_profile("last_active");
	*/
}

void ConnectBox::setparm (void)
{
	bottom.box=TRUE;
	bottom.setparm (xbase +3 , ybase + CONNBOX_HEIGHT,CONNBOX_WIDTH -6 ,1);
	bottom.llhook (ACS_BTEE);
	bottom.lrhook (ACS_BTEE);
	bottom.setvalue ("<C>Connbox");

	encsetbox.setparm(xbase, ybase);
	//streamsetbox.setparm();
	//showcurrval();
}

void ConnectBox::draw (void)
{

	bottom.draw();

	refresh();
}

void ConnectBox::refresh (void)
{
	refreshCDKScreen (cdkscreen);
}

void ConnectBox::erase (void)
{
	bottom.destroy();
	//encsetbox.cleanup();
}

void ConnectBox::activate (void)
{
	int quit=0, c;

	while (!quit)
	{
		//bottom.setvalue ("<C>Use fields' shortcuts, set ic</48>e<!48>cast, set l</48>a<!48>me, or e</48></B>x<!B><!48>it");
		bottom.setvalue ("<C>Set La</48>m<!48>e/</48>O<!48>GG, edit </48>s<!48>ervers or e</48>x<!48>it");
		refresh();
		c=getch();
		switch (c) {
			case 'x':
				quit=1;
				break;
			case 'm':
			case 'M':
				bottom.setvalue ("<C></48>S<!48>ave/</48>L<!48>oad profiles, </48>a<!48>pply");
				encsetbox.draw();
				c=getch();
				encsetbox.unreg();
				//icecast_menu();
				break;
			case 'o':
			case 'O':
				bottom.setvalue ("<C></48>S<!48>ave/</48>L<!48>oad profiles, </48>a<!48>pply");
				encsetbox.draw();
				c=getch();
				//lame_menu();
				break;
			default:
				//quit=tabentry(c);
				break;
		}
	}
}

	
ConnectBox::~ConnectBox ()
{
	warning ("ConnectBox::~ConnectBox objcnt = %d [BEGIN]", cdkscreen->objectCount); 

	//bottom.destroy();

	encsetbox.cleanup();

	warning ("ConnectBox::~ConnectBox objcnt = %d ", cdkscreen->objectCount); 
	if (cdkscreen) {
		destroyCDKScreen(cdkscreen);
		cdkscreen=NULL;
	}
	warning ("ConnectBox::~ConnectBox objcnt [END]"); 
}

