/* A NCURSES/CDK TUI (Text User Interface) for MuSE
 * Copyright (C) 2003 Luca 'rubik' Profico <rubik@olografix.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: streamsetbox.cpp,v 1.1 2003/02/22 21:47:32 rubik Exp $
 *
 */

#include "streamsetbox.h"
#include "muse_console.h"

StreamSetBox::StreamSetBox ()
{
	logtypelist[0] = strdup ("x-audiocast");
	logtypelist[1] = strdup ("icy");
}

void StreamSetBox::setscreen (CDKSCREEN *screen)
{
	cdkscreen = screen;
}

void StreamSetBox::setparm (int x, int y)
{
	xbase = x; ybase = y;
	host.setscreen (cdkscreen); host.box=FALSE;
	port.setscreen (cdkscreen); port.box=FALSE;
	mnt.setscreen (cdkscreen); mnt.box=FALSE;
	name.setscreen (cdkscreen); name.box=FALSE;
	url.setscreen (cdkscreen); url.box=FALSE;
	description.setscreen (cdkscreen); description.box=FALSE;
	logintype.setscreen (cdkscreen); logintype.box=FALSE;
	pass.setscreen (cdkscreen); pass.box=FALSE;
	label1.setscreen (cdkscreen);

	host.setparm (NULL, "</48>h<!48></32>ost:<!32>", 20, xbase + 2, ybase + 3, false, '_');
	port.setparm (NULL, "</32>p<!32></48>o<!48></32>rt:<!32>", 5, xbase + 32, ybase + 3, false, '_');
	mnt.setparm (NULL, "</48>m<!48></32>nt:<!32>", 15, xbase + 45, ybase + 3, false, '_');
	
	name.setparm (NULL, "</48>n<!48></32>ame:<!32>", 27, xbase + 2, ybase + 5, false, '_');
	url.setparm (NULL, "</48>u<!48></32>rl:<!32>", 22, xbase + 38, ybase + 5, false, '_');
	
	description.setparm (NULL, "</48>d<!48></32>escription:<!32>", 50, xbase + 2, ybase + 7, false, '_');
	
	logintype.setparm (NULL, "</32>logint<!32></48>y<!48></32>pe:<!32>", xbase + 1, ybase + 9, logtypelist, 2, 0, false);
	pass.setparm (NULL, "</48>p<!48></32>ass:<!32>", 18, xbase + 29, ybase + 9, false, '_', A_REVERSE, vHMIXED);
	label1.setparm (xbase, ybase,20, 1);
	label1.setvalue ("Stream settings:");

}

void StreamSetBox::draw (void)
{
	host.draw();
	port.draw();
	mnt.draw();
	name.draw();
	url.draw();
	description.draw();
	logintype.draw();
	pass.draw();

}

void StreamSetBox::erase (void)
{
	host.destroy();
	port.destroy();
	mnt.destroy();
	name.destroy();
	url.destroy();
	description.destroy();
	logintype.destroy();
	pass.destroy();
	label1.destroy();
}

int StreamSetBox::tabentry (int k)
{
	int retval = 0;
	int loop=0;

	while (!loop) {
		loop=1;

		switch (k) {
			case 'h': //host
				host.activate();
				if (host.lastkey() != 9) break;
			case 'o': //port
				port.activate();
				if (port.lastkey() != 9) break;
			case 'm': //mnt
				mnt.activate();
				if (mnt.lastkey() != 9) break;
			case 'n': //name
				name.activate();
				if (name.lastkey() != 9) break;
			case 'u': //url
				url.activate();
				if (url.lastkey() != 9) break;
			case 'd': //description
				description.activate();
				if (description.lastkey() != 9) break;
			case 'y': //logintype
				logintype.activate();
				if (logintype.lastkey() != 9) break;
			case 'p': //pass
				pass.activate();
				if (pass.lastkey() != 9) break;
			case NULL:
				k='h'; // start from first case
				loop = 0;
				break;
			default:
				warning ("k = %d", k);
				retval = 0;
				break;
		}
	}
	return retval;
}

void StreamSetBox::showcurrval (void)
{
	/*
	char temp[32];
	
	host.setvalue (mixer->ice.ip);
	snprintf (temp, 32, "%d", mixer->ice.port);
	port.setvalue (temp);
	mnt.setvalue (mixer->ice.mount);
	name.setvalue (mixer->ice.name);
	url.setvalue (mixer->ice.url);
	description.setvalue (mixer->ice.desc);
	logintype.setitem (mixer->ice.login);
	pass.setvalue (mixer->ice.pass);
	*/
}

StreamSetBox::~StreamSetBox ()
{
	label1.destroy();
}

